define([
    'jquery'
], function ($) {

    $.fn.lazyload = function (triggerNow) {
        var settings = $.extend($.fn.lazyload.defaults, window.FiloBlu.LazyLoadScript.params);
        var images = $(this).filter('img');

        // Counter for custom event when all the images are loaded.
        var imagesCount = images.length;
        if (imagesCount > 0) {

            var CurrentMediaQuery = $.fn.lazyload.getCurrentMediaQuery();
            
            if (triggerNow) {
                loadActualImages(images, settings, 'load');
            } else {

                // Load on page load.
                
                // Wait for document ready to load images to prevent issues with other JS (as AD scripts using document.write).

                loadActualImages(images, settings, 'load');

                // Load on scroll
                $(window).on('scroll', function (e) {
                    loadActualImages(images, settings, 'scroll');
                });
                // Load on resize
                $(window).on('resize', function (e) {
                    var MediaQuery = $.fn.lazyload.getCurrentMediaQuery();
                    if (CurrentMediaQuery != MediaQuery) {

                        var lazyloaders;
                        lazyloaders = $(".lazyload,.lazyload-container");
                        if (lazyloaders.length) {
                            lazyloaders.removeClass("loaded");
                            lazyloaders.removeAttr('data-loaded', '');
                            lazyloaders.removeAttr('data-display', '');
                            lazyloaders.removeAttr('data-current-mediaq', '');
                        }

                        // Triger custom event on mediaquery change
                        $.event.trigger({
                            type: "onMediaqueryChange",
                            CurrentMediaQuery: MediaQuery,
                            PreviousMediaQuery: CurrentMediaQuery
                        });
                        CurrentMediaQuery = MediaQuery;
                        loadActualImages(images, settings, 'resize');
                    }
                });
            }

        }

        function responsiveImagesDynamicHeight(image, callOrigin) {
            if (callOrigin == "load" || callOrigin == "resize") {
                var coeff,
                        originalHeight,
                        originalWidth;

                var parent = image.parent();
                if (parent.hasClass("lazyload-container")) {

                    //loop all the breakpoint
                    $.each($.fn.lazyload.defaults.breakpoint, function (key, val) {

                        if ($.fn.lazyload.is_valid_mediaQuery(val[1])) {
                            originalWidth = image.data('original-width-' + val[0]);
                            originalHeight = image.data('original-height-' + val[0]);

                        }
                    });

                    if (!originalWidth || !originalHeight) {
                        originalWidth = image.data('original-width');
                        originalHeight = image.data('original-height');
                    }


                    if (originalWidth && originalHeight) {
                        coeff = (originalHeight / originalWidth) * 100;

                        parent.css({
                            'position': 'relative',
                            'height': 0,
                            'padding-bottom': parseFloat(coeff).toFixed(1) + '%'
                        });
                        image.css({
                            'position': 'absolute',
                            'top': 0,
                            'left': 0,
                            'width': '100%',
                            'height': '100%'
                        });
                    }
                }
            }
        }

        /**
         * Set an attribute "data-display" to the image
         * The first valid display will be set.
         * The image or a parent element can have the gridClass
         * For advance config, You can specify a "parent" element.
         * In this case, the parent must have the gridClass.
         * @param {Object} image
         */
        function imagesSetDisplay(image) {
            var currentDisplay = 'defaults'; // Default display (fallback)
            for (display in settings.displays)
            {
                if ($(image).hasClass(settings.displays[display].gridClass)
                        || $(image).closest((settings.displays[display].parent != undefined) ? settings.displays[display].parent : '.' + settings.displays[display].gridClass).hasClass(settings.displays[display].gridClass)) {
                    currentDisplay = display;
                    break;
                }
            }
            $(image).attr('data-display', currentDisplay);
        }

        /**
         * imagesGetDisplay
         * Get the image SRC value using the proper rendition name
         * depending of the mediaquery & the image display.
         *
         * To override the default "data-src" attribute you can use :
         * data-src-{display}-{MediaQuery}
         * data-src-{display}
         * data-src-{MediaQuery}
         *
         * @return string image apropriate src value.
         */
        function imagesGetDisplay(image) {
            if (!$(image).attr('data-display')) {
                imagesSetDisplay(image);
            }
            var activeDisplay = $(image).attr('data-display');
            var currentDisplay = null;
            // If explicit display
            if (activeDisplay) {
                // Set the current display according to the current display and active mediaquery
                if (settings.displays[activeDisplay]) {
                    if (settings.displays[activeDisplay].display[CurrentMediaQuery]) {
                        currentDisplay = settings.displays[activeDisplay].display[CurrentMediaQuery];
                    } else if (settings.displays[activeDisplay].display.defaults) {
                        currentDisplay = settings.displays[activeDisplay].display.defaults;
                    }
                }
            }
            //Default displays
            if (!currentDisplay) {
                // Fallback on current media Query
                if (settings.displays.defaults.display[CurrentMediaQuery]) {
                    currentDisplay = settings.displays.defaults.display[CurrentMediaQuery];
                }
                // Global Fallback
                else {
                    currentDisplay = settings.displays.defaults.display.defaults;
                }
            }
            /*
             * Edit the image SRC attribute.
             */
            // Override the token By Display-Breakpoint.
            if ($(image).attr('data-src-' + activeDisplay + '-' + CurrentMediaQuery)) {
                return $(image).attr('data-src-' + activeDisplay + '-' + CurrentMediaQuery).replace(settings.token, currentDisplay);
            }
            // Override the token By Display.
            else if ($(image).attr('data-src-' + activeDisplay)) {
                return $(image).attr('data-src-' + activeDisplay).replace(settings.token, currentDisplay);
            }
            // Override the token By Breakpoint.
            else if ($(image).attr('data-src-' + CurrentMediaQuery)) {
                return $(image).attr('data-src-' + CurrentMediaQuery).replace(settings.token, currentDisplay);
            }
            // Use the token.
            else if ($(image).attr("data-src")) {
                return $(image).attr("data-src").replace(settings.token, currentDisplay);
            }
            // Global fallback, return current src
            else {
                return $(image).attr("src");
            }
            return null;
        }
        ;

        /**
         * Loading actual images
         *
         * @param images {jQuery<Image>} A jQuery set of images
         * @param settings {Object} the settings
         * @param [callOrigin] {String} Determines the origin of the call. Can be either 'load'|'scroll'|'resize'
         */
        function loadActualImages(images, settings, callOrigin) {
            images.each(function () {
                responsiveImagesDynamicHeight($(this), callOrigin);
                if ($(this).attr('data-current-mediaq') != CurrentMediaQuery || settings.force === true || $(this).attr('data-force-load')) {
                    if (windowView(this, settings) || settings.force === true || $(this).attr('data-force-load')) {
                        // Check if this is the first load or if the mediaquery has changed.
                        $(this).attr('data-current-mediaq', CurrentMediaQuery);
                        loadImage(this, callOrigin);
//                                    $(this).fadeIn(400);
                    }
                }
            });
        }
        ;

        /**
         * windowView
         * Check if the images are within the window view (top, bottom, left and right)
         * @param image {Object} The image object
         * @param settings {Object} The responsive lazyloading settings
         * @return bool
         */
        function windowView(image, settings) {
            var imgToDisplay = $(image);
            if (imgToDisplay.hasClass("lazyload-overflow")) {
                image = imgToDisplay.parent();
            }

            // window variables
            var windowHeight = $(window).height(),
                    windowWidth = $(window).width(),
                    windowTop = $(window).scrollTop() - settings['distance'],
                    windowBottom = windowTop + windowHeight + (settings['distance'] * 2),
                    windowLeft = $(window).scrollLeft() - settings['distance'],
                    windowRight = windowLeft + windowWidth + (settings['distance'] * 2),
                    // image variables
                    imageHeight = $(image).height(),
                    imageWidth = $(image).width(),
                    imageTop = $(image).offset().top - settings['distance'],
                    imageBottom = imageTop + imageHeight + settings['distance'],
                    imageLeft = $(image).offset().left - settings['distance'],
                    imageRight = imageLeft + imageWidth + settings['distance'];

            // This will return true if any corner of the image is within the "active" screen including the distance parameter.
            return (((windowBottom >= imageTop) && (windowTop <= imageTop)) || ((windowBottom >= imageBottom) && (windowTop <= imageBottom))) &&
                    (((windowRight >= imageLeft) && (windowLeft <= imageLeft)) || ((windowRight >= imageRight) && (windowLeft <= imageRight)));
        }
        ;

        /**
         * loadImage
         * Load the image and trigger appropriate events.
         * @param image {Image}
         * @param [callOrigin] {String} determines the origin of the call. Can be either 'load'|'scroll'|'resize'
         */
        function loadImage(image, callOrigin) {
            var ImageLoad = new Image();

            $(ImageLoad).one('load', function () {
                // IE8 compatibility, issue on imageload event :
                // Only show the image when the width & height are not empty (image really loaded).
                if ('naturalWidth' in this !== true || 'naturalHeight' in this !== true)
                {
//				$(this).hide().css('width', 'auto');
                    this.naturalWidth = $(this).width();
                    this.naturalHeight = $(this).height();
//				$(this).show();
                }
                // Trigger the responsiveImageLoaded custom event on image load.
                $.event.trigger({
                    type: "responsiveImageLoaded",
                    image: $(image)
                });
                // If first load, decrement the image counter for onAllImagesLoad callback.
                if ($(image).attr('data-loaded') == undefined) {
                    imagesCount -= 1;
                    $(image).attr('data-loaded', true);
                }
                /*
                 * fire success callback only if callOrigin param undefined (default) or
                 * if callOrigin value matches settings use cases
                 */
                if (callOrigin === undefined || callOrigin === 'load'
                        || (callOrigin === 'scroll' && settings.useScroll)
                        || (callOrigin === 'resize' && settings.useResize))
                {
                    // Call the callback here because we are certain the image is loaded here
                    $(image).attr("src", $(this).attr("src"));
                    settings.onImageShow.call($(image));
                }
                // All images have been loaded.
                if (imagesCount === 0)
                {
                    settings.onAllImagesLoad.call(images, {
                        query: CurrentMediaQuery
                    });
                    imagesCount = images.length;
                }
                $(this).off('error');

            }).one('error', function () {
                /*
                 * a load error might be fired when src is set to null
                 * in that case we don't want to register it as a 'real' image load
                 */
                if ($(this).attr('src') !== '')
                {
                    // If first load, decrement the image counter for onAllImagesLoad callback.
                    if ($(image).attr('data-loaded') == undefined) {
                        imagesCount -= 1;
                        $(image).attr('data-loaded', true);
                    }
                    settings.onImageError();
                }
            });
            $(ImageLoad).attr('src', null); // Prevent bad image refresh on desktop window resize.
            $(ImageLoad).attr('src', imagesGetDisplay(image)); // Get the appropriate rendition URL and display it.
        }
        ;
        return $(this);
    };

    /**
     * Return TRUE if the mediaquery passed is valid.
     * Use modernizr.mq as fallback for old browsers (if available).
     * Return false if mediaquery detection is unavailable.
     * @param string media A mediaquery string as a CCS @media
     * @return bool
     */
    $.fn.lazyload.is_valid_mediaQuery = function (media) {
        // MatchMedia support detection.
        if ("matchMedia" in window) {
            if (window.matchMedia(media).matches) {
                return true;
            } else {
                return false;
            }
        }
        // Fallback using Modernizr MatchMedia support detection.
        else if (Modernizr) { // Prevent error if Modernizr isn't availabe.
            if (Modernizr.mq(media)) {
                return true;
            } else {
                return false;
            }
        }
        // Return false if mediaquery detection is unavailable.
        return false;
    };
    /**
     * Return the current mediaquery.
     * In order to be consistent with the CSS, return the last valid mediaquery.
     * @return string the ID of the current "mediaQueries"
     */
    $.fn.lazyload.getCurrentMediaQuery = function () {
        var current = 'defaults';	//default fallback value
        var settings = $.fn.lazyload.defaults;
        for (m in settings.mediaQueries) {
            if ($.fn.lazyload.is_valid_mediaQuery(settings.mediaQueries[m])) {
                current = m;
            }
        }
        return current;
    };

    /**
     * Setup plugin defaults
     */
    $.fn.lazyload.defaults = {
        distance: 0, // the distance (in pixels) between the image and active window for loading the actual image
        force: false, // Force images loading without distance.
        useScroll: true, // (Dis)Allow settings.onImageShow() to be called on scroll.
        useResize: true, // (Dis)Allow settings.onImageShow() to be called after window resizes.
        token: '[[display]]', // The token replaced by the current display.
        mediaQueries: {// The mediaqueries used by the lazyloder.
            'phone': "(max-width: 460px)", // Phone default mediaQuery
            'tablet': "(min-width: 461px) and (max-width: 991px)", // Tablet default mediaQuery
            'desktop': "(min-width: 992px)" 							// Desktop default mediaQuery, not used on display but useful for external use of getCurrentMediaQuery function.
        },
        displays: {// The displays to combine grid system & mediaqueries
            'defaults': {
                'display': {
                    'tablet': 'tablet',
                    'phone': 'phone',
                    'defaults': 'desktop'
                }
            }
        },
        onImageShow: function () {
            var parentEl,
                    that;
            that = $(this);
            that.addClass('loaded');
            parentEl = that.parent();
            if (parentEl.length > 0 && parentEl.hasClass("lazyload-container") && !parentEl.hasClass("loaded")) {
                parentEl.addClass("loaded");
            }
            if (that.attr("data-bg-element-id")) {
                $(that.attr("data-bg-element-id")).css("background-image", "url('" + that.attr("src") + "')").addClass("loaded");
            }
        }, // Callback when an image has been lazy loaded.
        onImageError: function () {}, // Callback when an image could not be lazy loaded.
        onAllImagesLoad: function () {}			// Callback when all the images of the set are loaded.
    };

});